local addonName, a = ...
local L = a.Localize
local s = SpellFlashAddon
local c = BittensGlobalTables.GetTable("BittensSpellFlashLibrary")

local GetTime = GetTime
local UnitPowerMax = UnitPowerMax

local format = string.format
local min = math.min
local select = select
local tostring = tostring
local unpack = unpack

local rotation = {}
local rotationName = '?'

a.Rotations = {}

a.smashPending = 0
a.smashTime = 0

-- local resetDamageOverTime = function()
--    a.InitCombatTime = -1
--    a.InitDamage = -1
-- end

-- local DamageOverTime = function()
--    if a.InitDamage ~= -1 then
--       a.DamageRate = (c.GetTime() - a.InitCombatTime) / (c.GetHealth() - a.InitDamage)
--       a.InitCombatTime = c.GetTime()
--       a.InitDamage = c.GetHealth()
--       c.Debug("Damage: ", a.DamageRate)
--    else
--       a.InitCombatTime = c.GetTime()
--       a.InitDamage = c.GetHealth()
--    end
-- end

local monitorSmashPending = function(info)
   if c.InfoMatches(info, "Colossus Smash") then
      a.smashTime = GetTime()
      a.smashPending = 1
   end
end

-- local monitorSmashApplied = function(spellID)
--    if c.IdMatches(spellID, "Colossus Smash") then
--       a.smashPending = 0
--    end
-- end

function a.PreFlash()
   if a.smashPending > 0 then
      a.smashPending = GetTime() - a.smashTime
      if a.smashPending >= 20 then
         a.smashPending = 0
      end
   end

   a.CSIsUsable = select(1, s.UsableSpell("Colossus Smash")) and a.smashPending == 0
   a.ExeIsUsable = select(1,s.UsableSpell("Execute"))
   a.MSIsUsable = select(1, s.UsableSpell("Mortal Strike")) and c.GetCooldown("Mortal Strike") == 0
   a.Rage = c.GetPower()

   a.InExecute = s.HealthPercent() <= 20
   a.SuddenDeath = c.HasBuff("Sudden Death")
   a.Enraged = c.HasBuff("Enrage")

   if c.IsQueued("Berserker Rage") then
      a.Rage = a.Rage + 10
      a.Enraged = true
   elseif c.IsQueued("Charge") then
      a.Rage = a.Rage + 20
   elseif c.IsQueued("Bloodthirst", "Mortal Strike") then
      a.Rage = a.Rage + 10
   elseif c.IsQueued("Battle Shout", "Commanding Shout") then
      a.Rage = a.Rage + 20
   elseif c.IsQueued("Shield Slam") then
      if s.Buff(c.GetID("Sword and Board"), "player") then
         a.Rage = a.Rage + 25
      else
         a.Rage = a.Rage + 20
      end
   elseif c.IsQueued("Revenge") then
      a.Rage = a.Rage + 15
   end

   local max = UnitPowerMax("player")
   a.maxPower = UnitPowerMax("player")
   a.Rage = min(max, a.Rage)
   a.EmptyRage = max - a.Rage

   if c.IsCasting("Victory Rush", "Impending Victory") then
      a.VictoriousDuration = 0
   else
      a.VictoriousDuration = c.GetBuffDuration("Victorious")
   end

   a.Bloodbath = c.HasBuff("Bloodbath", false, false, true)
end

-------------------------------------------------------------------------- Arms
local tastePending = 0
a.rendAppliedTime = 0

a.Rotations.Arms = {
   Spec = 1,

   UsefulStats = { "Strength", "Melee Hit", "Crit", "Haste" },

   FlashInCombat = function()
      a.TasteStacks = c.GetBuffStack("Taste for Blood")
      if c.IsCasting("Mortal Strike") or GetTime() - tastePending < .8 then
         a.TasteStacks = a.TasteStacks + 2
      end

      c.FlashAll(
         "Recklessness for Arms",
         "Avatar",
         "Bloodbath",
         "Berserker Rage for Arms",
         "Heroic Leap",
         "Impending Victory for Heals, Optional",
         "Victory Rush for Heals, Optional",
         --      "Enraged Regeneration",
         "Pummel",
         "Disrupting Shout")
      if c.AoE then
         c.FlashAll("Cleave for Arms", "Sweeping Strikes")
         c.PriorityFlash(
            "Mortal Strike",
            "Bladestorm",
            "Shockwave",
            "Dragon Roar for Arms",
            "Thunder Clap",
            "Whirlwind for Arms",
            "Colossus Smash",
            --        "Overpower AoE",
            "Shout for Rage",
            --        "Storm Bolt",
            "Heroic Throw")
      else
         a.PreFlash()
         c.FlashAll("Rend" )
         c.PriorityFlash(
            "Colossus Smash for Arms",
            "Execute for Arms",
            "Storm Bolt for Arms",
            "Dragon Roar for Arms",
            "Shockwave",
            "Slam",
            "Mortal Strike",
            "Rend",
            "Whirlwind for Arms")
      end
   end,

   FlashAlways = function()
      c.FlashAll("Dps Stance", "Shout for Buff")
   end,

   CastSucceeded = function(info)
      if c.InfoMatches(info, "Rend") then
         if a.rendAppliedTime == 0 then
            a.rendAppliedTime = GetTime()
         end
      else
         monitorSmashPending(info)
      end
   end,

   -- Taste for Blood, each time Rend deals damage, you gain 3 rage.
   AuraApplied = function(spellID)
      if c.IdMatches(spellID, "Rend") then
         if (GetTime() - a.rendAppliedTime) >= 17 then
            a.rendAppliedTime = 0
         end
      end
   end,

   ExtraDebugInfo = function()
      --  print("Rend: ", s.ShowBuffs("target", "harmful"))
      --	print("[CS: ", a.CSIsUsable, " CS Pending: ", a.smashPending, " Rend: ", c.HasMyDebuff("Rend", false,false), " RUse: ", s.UsableSpell("Rend"), "]")
   end,
}

-------------------------------------------------------------------------- Fury

a.Rotations.Fury = {
   Spec = 2,

   UsefulStats = {"Multistrike", "Strength", "Crit", "Mastery", "Haste", "Versatility"},

   RotationAoE = {
      "Ravager for Fury",
      "Raging Blow with 3x Meat Cleaver and Enraged",
      "Bloodthirst during cleave",
      "Raging Blow with 3x Meat Cleaver",
      -- recklessness,sync=bladestorm -- already done in shared bit, I think
      "Bladestorm if Enraged for duration",
      "Whirlwind",
      "Execute with Sudden Death",
      "Dragon Roar",
      "Bloodthirst",
      "Wild Strike with Bloodsurge"
   },

   ThreeTargets = {
      "Ravager for Fury",
      "Bladestorm if Enraged",
      "Bloodthirst during cleave",
      "Raging Blow with 2x Meat Cleaver",
      "Execute",
      "Execute with Sudden Death",
      "Dragon Roar",
      "Whirlwind",
      "Bloodthirst",
      "Wild Strike with Bloodsurge"
   },

   TwoTargets = {
      "Ravager for Fury",
      "Dragon Roar",
      "Bladestorm if Enraged",
      "Bloodthirst during cleave",
      "Execute",
      "Execute with Sudden Death",
      "Raging Blow with Meat Cleaver",
      "Whirlwind for Meat Cleaver",
      "Wild Strike with Bloodsurge to avoid cap",
      "Bloodthirst",
      "Whirlwind near cap",
      "Wild Strike with Bloodsurge"
   },

   RotationST = {
      -- recklessness,if=target.health.pct<20&raid_event.adds.exists
      "Wild Strike at Cap",
      "Bloodthirst early",
      "Ravager for Fury",
      "Execute with Sudden Death",
      "Siegebreaker",
      "Storm Bolt",
      "Wild Strike with Bloodsurge",
      "Execute when Enraged",
      "Dragon Roar",
      "Raging Blow",
      "Wild Strike if Enraged",
      "Bladestorm",
      "Shockwave",
      "Impending Victory",
      "Bloodthirst"
   },

   FlashInCombat = function()
      -- @todo danielp 2015-01-18: revisit this and consider if it should be
      -- one "spell", or multiple of them, to make this work.
      c.FlashAll(
         "Shout",
         "Pummel",
         "Disrupting Shout"
      )

      -- @todo danielp 2015-01-18: handle other rotation decisions here
      if c.EstimatedHarmTargets > 3 then
         rotation = a.Rotations.Fury.RotationAoE
         rotationName = 'a'
      elseif c.EstimatedHarmTargets == 3 then
         rotation = a.Rotations.Fury.ThreeTargets
         rotationName = '3'
      elseif c.EstimatedHarmTargets == 2 then
         rotation = a.Rotations.Fury.TwoTargets
         rotationName = '2'
      else
         rotation = a.Rotations.Fury.RotationST
         rotationName = 's'
      end

      c.DelayPriorityFlash(
         "Impending Victory for Heals",
         "Victory Rush for Heals",
         "Charge",
         -- call_action_list,name=movement,if=movement.distance>5
         "Berserker Rage for Fury",
         -- heroic_leap,if=<moving>
         "Recklessness for Fury",
         "Avatar",
         "Bloodbath",
         unpack(rotation) -- must remain last line
      )
   end,


   --  FlashAlways = function()
   --    c.FlashAll("Dps Stance", "Shout for Buff")
   --  end,
   --
   --  CastSucceeded = function(info)
   --    if c.InfoMatches(info, "Whirlwind") then
   --      a.CleaverPending = GetTime()
   --    end
   --  end,
   --
   --  AuraApplied = function(spellID)
   --    if c.IdMatches(spellID, "Meat Cleaver") then
   --      a.CleaverPending = 0
   --    end
   --  end,
   --
   --  AuraRemoved = function(spellID)
   --    if c.IdMatches(spellID, "Meat Cleaver") then
   --      a.CleaverDumpPending = 0
   --    end
   --  end,

   FlashOutOfCombat = function()
      c.PriorityFlash(
         "Heroic Throw for Fury",
         "Charge"
      )
   end,

   ExtraDebugInfo = function()
      return format(
         "%s: r:%d ut:%s bs:%s",
         rotationName,
         a.Rage,
         tostring(c.HasTalent("Unquenchable Thirst")),
         tostring(c.HasBuff("Bloodsurge"))
      )
   end,
}

-------------------------------------------------------------------- Protection
local uncontrolledMitigationCooldowns = {
   "Spell Reflection",
}

a.Rotations.Protection = {
   Spec = 3,

   -- New Stat for Protection: Stamina > Versatility > Bonus Armor > Armor > Mastery > Haste > Crit > Multistrike > Strength
   -- The following list suggests the consumables to augment your warrior stats
   UsefulStats = { "Stamina", "Strength", "Mastery" },  --, "Armor" },

   FlashOutOfCombat = function()
      c.PriorityFlash(
         "Heroic Throw for Prot"
      )
   end,

   FlashInCombat = function()
      c.FlashMitigationBuffs(
         1,
         uncontrolledMitigationCooldowns,
         c.COMMON_TANKING_BUFFS,
         "No Mitigation if Victory Available",
         "Demoralizing Shout for Prot",
         "Enraged Regeneration for Prot",
         "Shield Wall under 3 min",
         "Last Stand for Prot",
         "Shield Block for Prot",
         "Shield Barrier for Prot",
         "Shield Wall 3+ min"
      )
      c.FlashAll(
         "Shield Block for Prot",
         "Shield Barrier for Prot",
         "Disrupting Shout",
         "Pummel"
      )
      --    c.DelayPriorityFlash(
      c.PriorityFlash(
         "Berserker Rage for Prot",
         "Bloodbath for Prot",
         "Execute for Prot",
         "Heroic Strike for Prot",
         "Shield Slam for Prot",
         "Revenge for Prot",
         "Ravager for Prot",
         "Storm Bolt for Prot",
         "Dragon Roar for Prot",
         "Impending Victory for Prot",
         "Devastate for Prot"
      )
   end,

   ExtraDebugInfo = function()
   end,
}
